@@ -119,9 +119,9 @@ rm_undocumented = function(pkg) {
 ##' @rdname roxygen_and_build
 ##' @export
 ##' @examples \dontrun{
-##' roxygen_and_build("./Rd2roxygen", install = TRUE)
+##' roxygen_and_build("Rd2roxygen", install = TRUE)
 ##' ## or simply
-##' rab('./Rd2roxygen', install = TRUE)
+##' rab('Rd2roxygen', install = TRUE)
 ##' }
 roxygen_and_build = function(pkg, roxygen.dir = NULL, install = FALSE,
     check = FALSE, check.opts = "", remove.check = TRUE, escape = TRUE,
@@ -136,6 +136,15 @@ roxygen_and_build = function(pkg, roxygen.dir = NULL, install = FALSE,
         unlink(inst.dir, recursive = TRUE)
     rm_undocumented(roxygen.dir)
     rd.list = list.files(file.path(roxygen.dir, "man"), ".*\\.Rd$", all.files = TRUE, full.names = TRUE)
+    if (reformat) {
+        for (f in rd.list) {
+            if (isTRUE(reformat_code(f))) {
+                x = readLines(f)
+                x = gsub('\\\\dontrun', '\\dontrun', x, fixed = TRUE)
+                writeLines(x, con = f)
+            }
+        }
+    }
     if (escape) {
         for (f in rd.list) {
             x = readLines(f)
@@ -146,14 +155,6 @@ roxygen_and_build = function(pkg, roxygen.dir = NULL, install = FALSE,
             }
         }
     }
-    if (reformat) {
-        for (f in rd.list) {
-            reformat_code(f)
-            x = readLines(f)
-            x = gsub('\\\\dontrun', '\\dontrun', x, fixed = TRUE)
-            writeLines(x, con = f)
-        }
-    }
     system(sprintf("R CMD build %s ", roxygen.dir))
     if (install)
         system(sprintf("R CMD INSTALL %s ", roxygen.dir))
@@ -172,33 +173,103 @@ rab = function(...) {
 
 
 ##' Format the code in the usage and examples sections.
+##' This function can polish the Rd files generated by roxygen mainly
+##' in two sections: usage and examples. By default, roxygen will omit
+##' the spaces and indent in the code, which makes the code
+##' (especially in the examples section) hard to read. It uses the
+##' function \code{\link[formatR]{tidy.source}} in the package
+##' \pkg{formatR}.
 ##'
-##' This function can polish the Rd files generated by roxygen mainly in two
-##' sections: usage and examples. By default, roxygen will omit the spaces and
-##' indent in the code, which makes the code (especially in the examples section)
-##' hard to read.
+##' If the macro \code{"\\dontrun"} is detected in the Rd file, this
+##' function will use an unstable way to reformat the example and
+##' usage sections, which might destroy your original Rd (e.g. the
+##' subsection macro can be mangled); otherwise it will use ordinary
+##' text-processing techniques to deal with these sections and
+##' generally will not affect other sections. However, neither way is
+##' absolutely safe. Whenever you run into troubles, just try to turn
+##' off reformatting.
 ##' @param path the path of the Rd file
-##' @param section the sections in the Rd file to (re)format
+##' @param section the sections in the Rd file to (re)format (the only
+##' possible choices are \code{"examples"} and \code{"usage"})
 ##' @param ... other arguments passed to \code{tidy.source}
-##' @return NULL (the original Rd file gets updated)
+##' @return This function returns \code{TRUE} if the macro
+##' \code{"\\dontrun"} is detected in the Rd; \code{FALSE} if not
+##' deteced; and \code{NULL} if the package \pkg{formatR} is not
+##' installed (as a side effect, the original Rd file will be updated)
 ##' @export
 ##' @author Yihui Xie <\url{http://yihui.name}>
+##' @seealso \code{\link[formatR]{tidy.source}}
 ##' @examples
-##' \dontrun{
 ##' rd.file = system.file('examples', 'reformat_code_demo.Rd', package = 'Rd2roxygen')
 ##' file.copy(rd.file, tempdir())
 ##' fmt.file = file.path(tempdir(), 'reformat_code_demo.Rd')
-##' file.show(fmt.file)
+##'
+##' if (interactive()) file.show(fmt.file)  ## show the raw Rd
 ##' reformat_code(fmt.file)
-##' file.show(fmt.file)
-##' }
+##' if (interactive()) file.show(fmt.file)  ## the formatted Rd
+##'
 reformat_code = function(path, section = c('examples', 'usage'), ...) {
     if (require('formatR', quietly = TRUE)) {
+        rd = readLines(path)
+        ## easy to deal with when there are no \\dontrun's
+        if (!any(grepl('\\\\dontrun', rd))) {
+            message('(*) good, I did not detect the \\dontrun macro in your Rd \n  ', path)
+            if ('usage' %in% section) {
+                idx0 = grep('^\\\\usage\\{', rd)
+                idx1 = grep('^\\\\description\\{', rd)
+                if (length(idx0) && length(idx1)) {
+                    message('  reformatting section usage')
+                    ## \description always comes after \usage
+                    idx1 = idx1 - 2
+                    ## users may provide \usage{f()} (the right bracket on the same line)
+                    if (flag <- idx1 < idx0) idx1 = idx0
+                    tmp = rd[idx0:idx1]
+                    tmp[1] = sub('^\\\\usage\\{', '', tmp[1])
+                    if (flag) tmp = sub('\\}[ ]*$', '', tmp)
+                    txt = paste(tidy.source(text = tmp, output = FALSE,
+                                keep.blank.line = TRUE, ...)$text.tidy, collapse = '\n')
+                    txt = paste('\\usage{', txt, sep = '')
+                    if (flag)
+                        rd[idx0] = paste(txt, '}', sep = '') else rd[idx0:(idx1 + 1)] =
+                            c(txt, rep('', idx1 - idx0), '}')
+                }
+            }
+            if ('examples' %in% section) {
+                ## remove trailing spaces
+                while (tail(rd, 1) == '') {
+                    rd = rd[-length(rd)]
+                }
+                idx0 = grep('^\\\\examples\\{', rd)
+                if (length(idx0)) {
+                    message('  reformatting section examples')
+                    ## only \alias might appear after \examples
+                    idx1 = grep('^\\\\alias\\{', rd)
+                    if (length(idx1) && any(idx1 > idx0))
+                        idx1 = min(idx1[idx1 > idx0]) - 1 else idx1 = length(rd)
+                    tmp = rd[idx0:idx1]
+                    tmp[1] = sub('^\\\\examples\\{', '', tmp[1])
+                    nn = length(tmp)
+                    tmp[nn] = sub('\\}$', '', tmp[nn])
+                    txt = tidy.source(text = tmp, output = FALSE,
+                                keep.blank.line = TRUE, ...)$text.tidy
+                    txt[1] = paste('\\examples{', txt[1], sep = '')
+                    nn0 = length(txt)
+                    txt[nn0] = paste(txt[nn0], '}', sep = '')
+                    txt = paste(txt, collapse = '\n')
+                    rd[idx0:idx1] = c(txt, rep('', idx1 - idx0))
+                }
+            }
+            writeLines(rd, path)
+            flush.console()
+            return(invisible(FALSE))
+        }
+        message('(!) reformatting might destroy your Rd without warnings,\n  because I detected the \\dontrun macro in \n  ', path)
         rd = tools::parse_Rd(path)
         flag = FALSE
         for (sec in section) {
             idx = which(sapply(rd, tag) == paste('\\', sec, sep = ''))
             if (length(idx)) {
+                message('  reformatting section ', sec)
                 txt = rd[idx]
                 class(txt) = 'Rd'
                 txt = as.character(txt)
@@ -214,15 +285,15 @@ reformat_code = function(path, section = c('examples', 'usage'), ...) {
                 txt = gsub('tag_name_dontrun = function() {', '\\dontrun{', txt, fixed = TRUE)
                 rd[[idx]] = structure(list(structure(txt, Rd_tag = 'TEXT')), Rd_tag = paste('\\', sec, sep = ''))
                 flag = TRUE
-                message('reformatted section ', sec, ' in ', path)
-            } else message('section ', sec, ' not found in ', path)
+            } else message('  section ', sec, ' not found')
         }
+        flush.console()
         if (flag) {
             class(rd) = 'Rd'
             cat(as.character(rd, deparse = TRUE), file = path, sep = '')
+            return(invisible(TRUE))
         }
     } else {
         message('unable to tidy the code: the package "formatR" was not installed')
     }
-    flush.console()
 }
